==> equinox-0.0.3.tar.gz: lib/components/select/select.dart <==
export 'package:equinox/components/select/select_item.dart';
import 'package:equinox/equinox.dart';
import 'package:equinox/equinox_internal.dart';
import 'dart:math';
import 'package:flutter/widgets.dart';

class EqSelect<T> extends StatefulWidget {
  final List<EqSelectItem<T>> items;
  final void Function(T) onSelect;
  final String label;
  final String hint;
  final String description;
  final IconData icon;
  final WidgetShape shape;
  final WidgetStatus status;
  final int selectedIndex;

  const EqSelect({
    Key key,
    @required this.items,
    @required this.onSelect,
    @required this.hint,
    this.label,
    this.description,
    this.icon,
    this.shape = WidgetShape.rectangle,
    this.status,
    this.selectedIndex,
  }) : super(key: key);

  @override
  _EqSelectState<T> createState() => _EqSelectState<T>();
}

class _EqSelectState<T> extends State<EqSelect>
    with SingleTickerProviderStateMixin {
  int selectedIndex;
  bool outlined = false;
  OverlayEntry _overlayEntry;
  final LayerLink _layerLink = LayerLink();

  AnimationController animationController;
  Animation<double> animation;
  Duration animationDuration;

  didChangeDependencies() {
    super.didChangeDependencies();
    var theme = EqTheme.of(context);
    animationDuration = EqTheme.of(context).minorAnimationDuration;

    if (animationController != null) {
      animationController.duration = animationDuration;
    } else {
      animationController =
          AnimationController(vsync: this, duration: animationDuration);
    }
    animation = CurvedAnimation(
        parent: animationController, curve: theme.minorAnimationCurve);
  }

  dispose() {
    animationController.dispose();
    super.dispose();
  }

  initState() {
    super.initState();
    selectedIndex = widget.selectedIndex ?? null;
  }

  Color _getBorderColor(EqThemeData theme) {
    return (widget.status != null)
        ? theme.getColorsForStatus(status: widget.status).shade500
        : theme.borderBasicColors.color3;
  }

  Color _getFocusedBorderColor(EqThemeData theme) {
    return (widget.status != null)
        ? theme.getColorsForStatus(status: widget.status).shade500
        : theme.primary.shade500;
  }

  showOverlay() {
    if (this._overlayEntry == null) {
      this._overlayEntry = createOverlay();
      Overlay.of(context).insert(this._overlayEntry);
      animationController.forward();
      setState(() {});
    }
  }

  hideOverlay() {
    if (this._overlayEntry != null) {
      animationController.reverse();

      Future.delayed(animationDuration, () {
        if (!mounted) return;

        this._overlayEntry.remove();
        this._overlayEntry = null;
        setState(() {});
      });
    }
  }

  toggleOverlay() {
    if (this._overlayEntry == null)
      showOverlay();
    else
      hideOverlay();
  }

  bool get _isOverlayOpen => _overlayEntry != null;

  OverlayEntry createOverlay() {
    RenderBox renderBox = context.findRenderObject();
    var size = renderBox.size;
    var theme = EqTheme.of(context);
    double verticalOffset = size.height - 2.0;
    if (widget.description != null)
      verticalOffset -= theme.paragraph2.lineHeight + 4.0;
    if (widget.label != null) verticalOffset -= theme.label.lineHeight + 6.0;
    var borderRadius = theme.borderRadius *
        WidgetShapeUtils.getMultiplier(shape: widget.shape);
    return OverlayEntry(
        builder: (context) => Positioned(
              width: size.width,
              child: CompositedTransformFollower(
                showWhenUnlinked: false,
                link: this._layerLink,
                offset: Offset(0.0, verticalOffset),
                child: AnimatedBuilder(
                  animation: animation,
                  builder: (context, _) => EqInternalCard(
                        borderRadius: BorderRadius.vertical(
                            bottom: Radius.circular(borderRadius)),
                        child: SizeTransition(
                          axis: Axis.vertical,
                          sizeFactor: animation,
                          child: ListView(
                            padding: EdgeInsets.zero,
                            shrinkWrap: true,
                            children: widget.items.map((item) {
                              int index = widget.items.indexOf(item);
                              return EqListItem(
                                title: item.title,
                                subtitle: item.subtitle,
                                icon: item.icon,
                                onTap: () {
                                  widget.onSelect(item.value);
                                  setState(() => selectedIndex = index);
                                  hideOverlay();
                                },
                                status: widget.status,
                                active: index == selectedIndex,
                              );
                            }).toList(),
                          ),
                        ),
                      ),
                ),
              ),
            ));
  }

  @override
  Widget build(BuildContext context) {
    var theme = EqTheme.of(context);
    var borderRadius = WidgetShapeUtils.getMultiplier(shape: widget.shape) *
        theme.borderRadius;

    var borderRadiusModified = BorderRadius.vertical(
      top: Radius.circular(borderRadius),
      bottom: (!_isOverlayOpen) ? Radius.circular(borderRadius) : Radius.zero,
    );

    var borderRadiusModifiedOutline = BorderRadius.vertical(
      top: Radius.circular(borderRadius),
      bottom: (!_isOverlayOpen)
          ? Radius.circular(borderRadius)
          : Radius.circular(theme.borderRadius),
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        if (widget.label != null) ...[
          Text(
            widget.label.toUpperCase(),
            style: theme.label.textStyle.copyWith(color: theme.textHintColor),
          ),
          SizedBox(height: 6.0),
        ],
        OutlinedGestureDetector(
          onOutlineChange: (v) => setState(() => outlined = v),
          onTap: toggleOverlay,
          child: OutlinedWidget(
            outlined: outlined || (_overlayEntry != null),
            borderRadius: borderRadiusModifiedOutline,
            child: CompositedTransformTarget(
              link: _layerLink,
              child: AnimatedContainer(
                duration: theme.minorAnimationDuration,
                curve: theme.minorAnimationCurve,
                decoration: BoxDecoration(
                  borderRadius: borderRadiusModified,
                  color: theme.backgroundBasicColors.color2,
                  border: Border.all(
                    width: 1.0,
                    color: _isOverlayOpen
                        ? _getFocusedBorderColor(theme)
                        : _getBorderColor(theme),
                  ),
                ),
                alignment: Alignment.centerLeft,
                padding: const EdgeInsets.only(
                  left: 18.0,
                  right: 12.0,
                  top: 8.0,
                  bottom: 8.0,
                ),
                child: Row(
                  mainAxisSize: MainAxisSize.min,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Expanded(
                      child: Text(
                        selectedIndex != null
                            ? widget.items[selectedIndex].title
                            : widget.hint,
                        style: (selectedIndex != null)
                            ? theme.subtitle1.textStyle
                            : theme.paragraph1.textStyle.copyWith(
                                color: theme.textHintColor,
                              ),
                      ),
                    ),
                    SizedBox(width: 16.0),
                    AnimatedBuilder(
                      animation: animation,
                      builder: (context, _) {
                        return Transform.rotate(
                          angle: (animation.value) * pi,
                          child: Icon(EvaIcons.chevronDown,
                              color: theme.textHintColor),
                        );
                      },
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
        if (widget.description != null) ...[
          SizedBox(height: 4.0),
          Text(
            widget.description,
            style:
                theme.paragraph2.textStyle.copyWith(color: theme.textHintColor),
          ),
        ],
      ],
    );
  }
}

==> equinox-0.0.6.tar.gz: lib/components/select/select.dart <==
export 'package:equinox/components/select/select_item.dart';
import 'package:equinox/equinox.dart';
import 'package:equinox/equinox_internal.dart';
import 'package:flutter/material.dart';
import 'dart:math';
import 'package:equinox/components/select/select_overlay.dart';
import 'package:flutter/widgets.dart';

class EqSelect<T> extends StatefulWidget {
  final List<EqSelectItem<T>> items;
  final void Function(T) onSelect;
  final String label;
  final String hint;
  final String description;
  final IconData icon;
  final WidgetShape shape;
  final WidgetStatus status;
  final int selectedIndex;

  const EqSelect({
    Key key,
    @required this.items,
    @required this.onSelect,
    @required this.hint,
    this.label,
    this.description,
    this.icon,
    this.shape = WidgetShape.rectangle,
    this.status,
    this.selectedIndex,
  }) : super(key: key);

  @override
  _EqSelectState<T> createState() => _EqSelectState<T>();
}

class _EqSelectState<T> extends State<EqSelect>
    with SingleTickerProviderStateMixin {
  int selectedIndex;
  bool outlined = false;
  OverlayEntry _overlayEntry;
  final LayerLink _layerLink = LayerLink();

  AnimationController animationController;
  Animation<double> animation;
  Duration animationDuration;
  bool openingFromBottom = null;

  didChangeDependencies() {
    super.didChangeDependencies();
    var theme = EqTheme.of(context);
    animationDuration = EqTheme.of(context).minorAnimationDuration;

    if (animationController != null) {
      animationController.duration = animationDuration;
    } else {
      animationController =
          AnimationController(vsync: this, duration: animationDuration);
    }
    animation = CurvedAnimation(
        parent: animationController, curve: theme.minorAnimationCurve);
  }

  dispose() {
    animationController.dispose();
    super.dispose();
  }

  initState() {
    super.initState();
    selectedIndex = widget.selectedIndex ?? null;
  }

  Color _getBorderColor(EqThemeData theme) {
    return (widget.status != null)
        ? theme.getColorsForStatus(status: widget.status).shade500
        : theme.borderBasicColors.color3;
  }

  Color _getFocusedBorderColor(EqThemeData theme) {
    return (widget.status != null)
        ? theme.getColorsForStatus(status: widget.status).shade500
        : theme.primary.shade500;
  }

  showOverlay() {
    if (this._overlayEntry == null) {
      this._overlayEntry = createOverlay();
      Overlay.of(context).insert(this._overlayEntry);
      animationController.forward();
      setState(() {});
    }
  }

  hideOverlay() {
    if (this._overlayEntry != null) {
      animationController.reverse();

      Future.delayed(animationDuration, () {
        if (!mounted) return;

        this._overlayEntry.remove();
        this._overlayEntry = null;
        this.openingFromBottom = null;
        setState(() {});
      });
    }
  }

  toggleOverlay() {
    if (this._overlayEntry == null)
      showOverlay();
    else
      hideOverlay();
  }

  bool get _isOverlayOpen => _overlayEntry != null;

  OverlayEntry createOverlay() {
    RenderBox renderBox = context.findRenderObject();
    var size = renderBox.size;
    var theme = EqTheme.of(context);

    double containerHeight = size.height;
    if (widget.description != null)
      containerHeight -= theme.paragraph2.lineHeight + 4.0;

    if (widget.label != null) containerHeight -= theme.label.lineHeight + 6.0;

    double verticalOffset = containerHeight;
    var borderRadius = theme.borderRadius *
        WidgetShapeUtils.getMultiplier(shape: widget.shape);


    Size screen = MediaQuery.of(context).size;

    var height = 0.0;
    for (var item in widget.items) {
      height += item.caluclateHeight(theme);
    }

    var position = renderBox.localToGlobal(Offset.zero);

    var bottom = screen.height - position.dy - size.height;
    var top = position.dy;

    openingFromBottom = true;

    if (height > bottom && height < top) {
      verticalOffset = -height;
      openingFromBottom = false;
    } else if (height > bottom && height > top) {
      openingFromBottom = true;
      height = bottom - 16.0;
    }

    return OverlayEntry(
      builder: (context) => Positioned(
            width: size.width,
            child: CompositedTransformFollower(
              showWhenUnlinked: false,
              link: this._layerLink,
              offset: Offset(0.0, verticalOffset),
              child: Container(
                height: height,
                alignment: (openingFromBottom)
                    ? Alignment.topCenter
                    : Alignment.bottomCenter,
                child: EqSelectOverlay(
                  openingFromBottom: openingFromBottom,
                  animation: animation,
                  borderRadius: borderRadius,
                  items: widget.items,
                  onSelect: (index, item) {
                    widget.onSelect(item.value);
                    setState(() => selectedIndex = index);
                    hideOverlay();
                  },
                  selectedIndex: selectedIndex,
                  status: widget.status,
                  theme: theme,
                ),
              ),
            ),
          ),
    );
  }

  @override
  Widget build(BuildContext context) {
    var theme = EqTheme.of(context);
    var borderRadius = WidgetShapeUtils.getMultiplier(shape: widget.shape) *
        theme.borderRadius;

    var normalRadius = Radius.circular(borderRadius);
    var topRadius = (!_isOverlayOpen)
        ? normalRadius
        : (!openingFromBottom) ? Radius.zero : normalRadius;
    var bottomRadius = (!_isOverlayOpen)
        ? normalRadius
        : (!openingFromBottom) ? normalRadius : Radius.zero;

    var borderRadiusModified = BorderRadius.vertical(
      top: topRadius,
      bottom: bottomRadius,
    );

    var borderRadiusModifiedOutline = BorderRadius.vertical(
      top: topRadius == Radius.zero ? Radius.circular(borderRadius) : topRadius,
      bottom: bottomRadius == Radius.zero
          ? Radius.circular(borderRadius)
          : bottomRadius,
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        if (widget.label != null) ...[
          Text(
            widget.label.toUpperCase(),
            style: theme.label.textStyle.copyWith(color: theme.textHintColor),
          ),
          SizedBox(height: 6.0),
        ],
        OutlinedGestureDetector(
          onOutlineChange: (v) => setState(() => outlined = v),
          onTap: toggleOverlay,
          child: OutlinedWidget(
            outlined: outlined || (_overlayEntry != null),
            borderRadius: (openingFromBottom == null)
                ? borderRadiusModified
                : borderRadiusModifiedOutline,
            child: CompositedTransformTarget(
              link: _layerLink,
              child: AnimatedContainer(
                duration: theme.minorAnimationDuration,
                curve: theme.minorAnimationCurve,
                decoration: BoxDecoration(
                  borderRadius: borderRadiusModified,
                  color: theme.backgroundBasicColors.color2,
                  border: Border.all(
                    width: 1.0,
                    color: _isOverlayOpen
                        ? _getFocusedBorderColor(theme)
                        : _getBorderColor(theme),
                  ),
                ),
                alignment: Alignment.centerLeft,
                padding: const EdgeInsets.only(
                  left: 18.0,
                  right: 12.0,
                  top: 8.0,
                  bottom: 8.0,
                ),
                child: Row(
                  mainAxisSize: MainAxisSize.min,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Expanded(
                      child: Text(
                        selectedIndex != null
                            ? widget.items[selectedIndex].title
                            : widget.hint,
                        style: (selectedIndex != null)
                            ? theme.subtitle1.textStyle
                            : theme.paragraph1.textStyle.copyWith(
                                color: theme.textHintColor,
                              ),
                      ),
                    ),
                    SizedBox(width: 16.0),
                    AnimatedBuilder(
                      animation: animation,
                      builder: (context, _) {
                        return Transform.rotate(
                          angle: (animation.value) * pi,
                          child: Icon(EvaIcons.chevronDown,
                              color: theme.textHintColor),
                        );
                      },
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
        if (widget.description != null) ...[
          SizedBox(height: 4.0),
          Text(
            widget.description,
            style:
                theme.paragraph2.textStyle.copyWith(color: theme.textHintColor),
          ),
        ],
      ],
    );
  }
}
